
;; ---------------------------------------------------------------------------
;; Miscellaneous emacs auxiliary code for lpdoc installation
;; ---------------------------------------------------------------------------

(setq max-lisp-eval-depth 3000)

;; Useful for rewriting and istallation of SETTINGS file
(defun fix-config-file-settings ()
  "Replace library path in SETTINGS during installation"
  (setq infile (my-get-option command-line-args "-infile"))
  (setq outfile (my-get-option command-line-args "-outfile"))
  (setq variable (my-get-option command-line-args "-variable"))
  (setq value (my-get-option command-line-args "-value"))
  (message (concat "Rewriting " infile " into " outfile "..."))
  (find-file infile)
  (search-forward-regexp (concat "^ *" variable "*=") nil t)
  (set-mark (point))
  (end-of-line)
  (kill-region (mark) (point))
  (insert-string value)
  (write-file outfile)
  (kill-emacs)
  )
  
;; Hook to run texinfo-multiple-files-update in batch mode
(load-library "texinfo")

(defun update-all-nodes-in-one-file ()
  "A hook to create a single, resolved .texi file in batch mode"
  (setq file (my-get-option command-line-args "-file"))
  (message (concat "*** Updating nodes in " file " ..."))
  (create-single-file file)
  (texinfo-all-menus-update t)
  (save-file-as-texi file)
  (kill-emacs t))

(defun create-single-file (file)
  "Creates a single file inserting the includes"
  (find-file file)
  (beginning-of-buffer)
  (insert-includes))

(defun insert-includes ()
  (setq start (point))
  (end-of-buffer)
  (setq buffer-limit (point))
  (goto-char start)
  (if (eq (search-forward "@include " buffer-limit t) nil)
      nil
    (beginning-of-line)
    (setq tmp (point))
    (search-forward " " buffer-limit t)
    (kill-region tmp (point))
    (setq tmp (point))
    (end-of-line)
    (setq compname (buffer-substring tmp (point)))
    (kill-region tmp (point))
    (insert-file compname) 
    (goto-char (mark))
    (message (concat "Inserted file " compname "..."))
    (insert-includes)))

(defun update-all-nodes ()
  "A hook to run texinfo-multiple-files-update in batch mode"
  (setq file (my-get-option command-line-args "-file"))
  (setq components (my-get-option command-line-args "-components"))
  (message 
   (concat "*** Updating all nodes for " file " with components " 
	   components " ..."))
;; Changed to cope properly with References: now always multiple files.
;;   (if (string= components nil)
;;       (progn 
;; 	(find-file file)
;; 	(texinfo-all-menus-update t))
      (texinfo-multiple-files-update file t t)
      (save-files-as-ltxi (gather-includes file "ltxi"))
;;      )
  (save-file-as-ltxi file)
  (kill-emacs t))

(defun gather-includes (file newsuffix)
  (find-file file)
  (end-of-buffer)
  (setq buffer-limit (point))
  (beginning-of-buffer)
  (get-includes buffer-limit newsuffix))

(defun get-includes (buffer-limit newsuffix)
  "Gets includes from main file and modifies includes"
  (if (eq (search-forward "@include " buffer-limit t) nil)
      nil
    (set-mark (point))
    (search-forward ".texic" buffer-limit t)
    (setq el (buffer-substring (mark) (point)))
    ;; Made more silent...
    ;; (message (concat "*** Component: " el))
    (set-mark (point))
    (backward-char 4)
    (kill-region (mark) (point))
    (insert-string newsuffix)
    (cons el (get-includes buffer-limit newsuffix))))

(defun save-files-as-ltxi (files)
  (if (eq files nil)
      (message "All component files saved")
    (save-file-as-ltxi (car files))
    (save-files-as-ltxi (cdr files))))

(defun save-file-as-ltxi (file)
  (find-file file)
;;  (message (concat "*** Saving " file " as .ltxi..."))
;;  (sleep-for 1)
  (write-file (fxname (buffer-name) "\\.texic" ".ltxi")))

(defun save-file-as-texi (file)
  (find-file file)
  (write-file (fxname (buffer-name) "\\.texic" ".texi")))

(defun fxname (iname suff1 suff2)
  (concat 
   (substring iname 0 (string-match suff1 iname))
   suff2))

;; Hook to fix references in .texic files
(defun update-all-references ()
  "A hook to run texinfo-multiple-files-update in batch mode"
  (setq file (my-get-option command-line-args "-file"))
  (setq refsfile (my-get-option command-line-args "-refsfile"))
  (setq components (my-get-option command-line-args "-components"))
  (message 
   (concat "*** Updating references for " file " with components " 
	   components " etc. ..."))
  (load-file refsfile)
  (update-references-in-files (gather-includes file "texic"))
  (kill-emacs t))

(defun update-references-in-files (files)
  (if (eq files nil)
      (message "All references processed")
    (update-references-in-file (car files))
    (update-references-in-files (cdr files))
    ))

(defun update-references-in-file (file)
  (if (eq (file-size (fxname file ".texic" ".refs")) 0)
      nil
    (message (concat "*** Updating references in " file "..."))
    (find-file file)
    (update-references nil)
    (save-buffer)))

(defun file-size (file)
  (nth 7 (file-attributes file)))

(defun refsfile (file)
  (refsfile file))

(defun update-references (insidelist)
  (if insidelist
      (progn 
	(forward-char 1)
	(update-reference)
	)
    (if (not (search-forward "[BibRef: " nil t))
	t
      (goto-char (match-beginning 0))
      (forward-char 1)
      (set-mark (point))
      (search-forward " ")
      (kill-region (point) (mark))
      (update-reference))))
	  
(defun update-reference ()
  (set-mark (point))
  (if (not (search-forward-regexp "\\(]\\|,\\)" nil t)) 
      (message "Error, could not find closing ']' or ',' after reference")
    (let 
	(keyword)
      (backward-char 1)
      (setq keyword (buffer-substring (point) (mark)))
      (kill-region (mark) (point))
      (insert-string (get-keyword-reference keyword))
      (if (equal (char-to-string (char-after (point))) ",")
	  (update-references t)
	(update-references nil))
      )))

(defun get-keyword-reference (keyword)
  (let 
      (reference)
    (setq reference (assoc keyword refs-alist))
    (if (eq reference nil)
	(concat "BibRef: " keyword)
       (car (cdr reference))
    )))

;; Command line option access
(defun my-get-option (command-line-args option)
  "Get the filename that appears in the command line after -option."
  (cond 
   ((eq command-line-args nil)
    (message (concat "Error: '" option "' argument not found"))
    (kill-emacs))
   ((string= (car command-line-args) option)
	(car (cdr command-line-args)))
   (t
    (my-get-option (cdr command-line-args) option))))

;; ---------------------------------------------------------------------------
  

